@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.ImportTool
@@ -19,6 +20,7 @@ module IDE.ImportTool (
 ,   addPackage
 ,   parseNotInScope
 ,   parseHiddenModule
+,   HiddenModuleResult(..)
 ) where
 
 import IDE.Core.State
@@ -53,7 +55,9 @@ import Distribution.PackageDescription.Parse
 import Distribution.Verbosity (normal)
 import IDE.Pane.PackageEditor (hasConfigs)
 import Distribution.Package
-import Distribution.Version (anyVersion)
+import Distribution.Version
+       (anyVersion, orLaterVersion, intersectVersionRanges,
+        earlierVersion, Version(..))
 import Distribution.PackageDescription
        (CondTree(..), condExecutables, condLibrary, packageDescription,
         buildDepends)
@@ -149,8 +153,8 @@ addPackage error = do
             ideMessage Normal $ "addPackage " ++ (display $ pkgName pack)
 #if MIN_VERSION_Cabal(1,10,0)
             liftIO $ writeGenericPackageDescription (ipdCabalFile $ idePackage)
-                gpd { condLibrary     = addDepToLib (packageName pack)  (condLibrary gpd),
-                      condExecutables = map (addDepToExe (packageName pack))
+                gpd { condLibrary     = addDepToLib pack (condLibrary gpd),
+                      condExecutables = map (addDepToExe pack)
                                             (condExecutables gpd)}
             return True
 #else
@@ -159,16 +163,25 @@ addPackage error = do
                 else do
                     let flat = flattenPackageDescription gpd
                     liftIO $ writePackageDescription (ipdCabalFile $ idePackage)
-                        flat { buildDepends =
-                            Dependency (pkgName pack) anyVersion : buildDepends flat}
+                        flat { buildDepends = dep pack : buildDepends flat}
                     return True
 #endif
   where
-    addDepToLib n Nothing = Nothing
-    addDepToLib n (Just cn@CondNode{condTreeConstraints = deps}) =
-        Just (cn{condTreeConstraints = (Dependency n anyVersion) : deps})
-    addDepToExe n (str,cn@CondNode{condTreeConstraints = deps}) =
-        (str,cn{condTreeConstraints = (Dependency n anyVersion) : deps})
+    addDepToLib _ Nothing = Nothing
+    addDepToLib p (Just cn@CondNode{condTreeConstraints = deps}) =
+        Just (cn{condTreeConstraints = dep p : deps})
+    addDepToExe p (str,cn@CondNode{condTreeConstraints = deps}) =
+        (str,cn{condTreeConstraints = dep p : deps})
+    -- Empty version is probably only going to happen for ghc-prim
+    dep p | null . versionBranch $ packageVersion p = Dependency (packageName p) (anyVersion)
+    dep p = Dependency (packageName p) (
+        intersectVersionRanges (orLaterVersion (packageVersion p))
+                               (earlierVersion (majorAndMinor (packageVersion p))))
+
+    majorAndMinor v@Version{versionBranch = b} = v{versionBranch = nextMinor b}
+    nextMinor = nextMinor' . (++[0,0])
+    nextMinor' (major:minor:_) = [major, minor+1]
+    nextMinor' _ = undefined
 
 getScopeForActiveBuffer :: IDEM (Maybe (GenScope, GenScope))
 getScopeForActiveBuffer = do
@@ -226,7 +239,7 @@ addImport' nis filePath descr descrList continuation =  do
                                                 fileSave False
                                                 setModified gtkbuf True
                                                 continuation (True,(descr : descrList))
-                         ServerFailed string	-> do
+                         ServerFailed string -> do
                             ideMessage Normal ("Can't parse module header " ++ filePath ++
                                     " failed with: " ++ string)
                             continuation (False,[])
@@ -389,13 +402,6 @@ selectModuleDialog parentWindow list id mbQual mbDescr =
                                                 Just pm -> (render . disp . modu) pm == v) list)))
                 _                      -> return Nothing
 
---testString =    "    Could not find module `Graphics.UI.Gtk':\n"
---             ++ "      It is a member of the hidden package `gtk-0.11.0'.\n"
---             ++ "      Perhaps you need to add `gtk' to the build-depends in your .cabal file.\n"
---             ++ "      Use -v to see a list of the files searched for."
---
---test = parseHiddenModule testString == Just (HiddenModuleResult {hiddenModule = "Graphics.UI.Gtk", missingPackage = PackageIdentifier {pkgName = PackageName "gtk", pkgVersion = Version {versionBranch = [0,11,0], versionTags = []}}})
-
 data HiddenModuleResult = HiddenModuleResult {
         hiddenModule      :: String
     ,   missingPackage    :: PackageId}
@@ -415,7 +421,8 @@ hiddenModuleParser = do
     whiteSpace
     symbol "Could not find module `"
     mod    <- many (noneOf "'")
-    symbol "':\n"
+    many (noneOf "\n")
+    symbol "\n"
     whiteSpace
     symbol "It is a member of the hidden package `"
     pack   <- many (noneOf "'")
