@@ -54,9 +54,7 @@ module IDE.Package (
 
 ,   printEvldWithShowFlag
 ,   tryDebug
-,   tryDebug_
 ,   tryDebugQuiet
-,   tryDebugQuiet_
 ,   executeDebugCommand
 
 ,   choosePackageFile
@@ -302,6 +300,7 @@ packageInstallDependencies = do
     lift $ catchIDE (do
         let dir = dropFileName (ipdCabalFile package)
         runExternalTool "Installing" "cabal" (["install","--only-dependencies"]
+            ++ (ipdConfigFlags package)
             ++ (ipdInstallFlags package)) (Just dir) logOutput)
         (\(e :: SomeException) -> putStrLn (show e))
 
@@ -396,7 +395,6 @@ packageOpenDoc = do
         let path = dropFileName (ipdCabalFile package)
                         </> "dist/doc/html"
                         </> display (pkgName (ipdPackageId package))
-                        </> display (pkgName (ipdPackageId package))
                         </> "index.html"
             dir = dropFileName (ipdCabalFile package)
         runExternalTool "Opening Documentation" (browser prefs) [path] (Just dir) logOutput)
@@ -404,16 +402,16 @@ packageOpenDoc = do
 
 runExternalTool :: String -> FilePath -> [String] -> Maybe FilePath -> E.Iteratee ToolOutput IDEM () -> IDEAction
 runExternalTool description executable args mbDir handleOutput = do
-        prefs          <- readIDE prefs
-        alreadyRunning <- isRunning
-        unless alreadyRunning $ do
-            when (saveAllBeforeBuild prefs) (do fileSaveAll belongsToWorkspace; return ())
-            triggerEventIDE (StatusbarChanged [CompartmentState description, CompartmentBuild True])
-            reifyIDE $ \ideR -> forkIO $ do
-                (output, pid) <- runTool executable args mbDir
-                reflectIDE (modifyIDE_ (\ide -> ide{runningTool = Just pid})) ideR
-                E.run_ $ output $$ (reflectIDEI handleOutput ideR)
-            return ()
+    prefs          <- readIDE prefs
+    alreadyRunning <- isRunning
+    unless alreadyRunning $ do
+        when (saveAllBeforeBuild prefs) (do fileSaveAll belongsToWorkspace; return ())
+        triggerEventIDE (StatusbarChanged [CompartmentState description, CompartmentBuild True])
+        reifyIDE $ \ideR -> forkIO $ do
+            (output, pid) <- runTool executable args mbDir
+            reflectIDE (modifyIDE_ (\ide -> ide{runningTool = Just pid})) ideR
+            E.run_ $ output $$ (reflectIDEI handleOutput ideR)
+        return ()
 
 
 -- ---------------------------------------------------------------------
@@ -711,13 +709,13 @@ debugStart = do
                 return ())
             (\(e :: SomeException) -> putStrLn (show e))
 
-tryDebug :: DebugM a -> PackageM (Maybe a)
+tryDebug :: DebugAction -> PackageAction
 tryDebug f = do
     maybeDebug <- lift $ readIDE debugState
     case maybeDebug of
         Just debug -> do
             -- TODO check debug package matches active package
-            liftM Just $ lift $ runDebug f debug
+            lift $ runDebug f debug
         _ -> do
             window <- lift $ getMainWindow
             resp <- liftIO $ do
@@ -734,25 +732,19 @@ tryDebug f = do
                     debugStart
                     maybeDebug <- lift $ readIDE debugState
                     case maybeDebug of
-                        Just debug -> liftM Just $ lift $ runDebug f debug
-                        _ -> return Nothing
-                _  -> return Nothing
+                        Just debug -> lift $ postAsyncIDE $ runDebug f debug
+                        _ -> return ()
+                _  -> return ()
 
-tryDebug_ :: DebugM a -> PackageAction
-tryDebug_ f = tryDebug f >> return ()
-
-tryDebugQuiet :: DebugM a -> PackageM (Maybe a)
+tryDebugQuiet :: DebugAction -> PackageAction
 tryDebugQuiet f = do
     maybeDebug <- lift $ readIDE debugState
     case maybeDebug of
         Just debug -> do
             -- TODO check debug package matches active package
-            liftM Just $ lift $ runDebug f debug
+            lift $ runDebug f debug
         _ -> do
-            return Nothing
-
-tryDebugQuiet_ :: DebugM a -> PackageAction
-tryDebugQuiet_ f = tryDebugQuiet f >> return ()
+            return ()
 
 executeDebugCommand :: String -> (E.Iteratee ToolOutput IDEM ()) -> DebugAction
 executeDebugCommand command handler = do
