@@ -1,5 +1,5 @@
-{-# OPTIONS_GHC -XRecordWildCards -XTypeSynonymInstances -XMultiParamTypeClasses
-    -XDeriveDataTypeable #-}
+{-# LANGUAGE FlexibleInstances, RecordWildCards, TypeSynonymInstances,
+             MultiParamTypeClasses, DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.Errors
@@ -25,7 +25,6 @@ module IDE.Pane.Errors (
 import Graphics.UI.Gtk
 import Data.Typeable (Typeable(..))
 import IDE.Core.State
-import Control.Monad.Reader
 import Graphics.UI.Gtk.General.Enums
     (Click(..), MouseButton(..))
 import Graphics.UI.Gtk.Gdk.Events (Event(..))
@@ -34,6 +33,7 @@ import IDE.ImportTool
         resolveErrors)
 import Data.List (elemIndex)
 import IDE.LogRef (showSourceSpan)
+import Control.Monad.IO.Class (MonadIO(..))
 
 -- | A breakpoints pane description
 --
