@@ -1,5 +1,5 @@
-{-# OPTIONS_GHC -XDeriveDataTypeable -XMultiParamTypeClasses -XTypeSynonymInstances
-    -XRecordWildCards #-}
+{-# LANGUAGE CPP, FlexibleInstances, DeriveDataTypeable, MultiParamTypeClasses,
+             TypeSynonymInstances, RecordWildCards #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.Grep
@@ -22,22 +22,37 @@ module IDE.Pane.Grep (
 ) where
 
 import Graphics.UI.Gtk hiding (get)
+import qualified Graphics.UI.Gtk.Gdk.Events as Gdk
 import Text.ParserCombinators.Parsec.Language
 import Text.ParserCombinators.Parsec hiding(Parser)
 import qualified Text.ParserCombinators.Parsec.Token as P
 import Data.Maybe
-import Control.Monad.Reader
 import Data.Typeable
 import IDE.Core.State
-import IDE.Utils.Tool (runTool, ToolOutput(..))
+import IDE.BufferMode
+import IDE.Utils.Tool (runTool, ToolOutput(..), getProcessExitCode, interruptProcessGroupOf)
 import Control.Concurrent
-       (newEmptyMVar, isEmptyMVar, takeMVar, putMVar, MVar, forkIO)
-import IDE.LogRef (logOutput)
+       (forkOS, newEmptyMVar, isEmptyMVar, takeMVar, putMVar, MVar,
+        forkIO)
+import IDE.LogRef (logOutput, defaultLineLogger)
 import IDE.Pane.SourceBuffer
-    (goToSourceDefinition)
+    (goToSourceDefinition, maybeActiveBuf, IDEBuffer(..))
+import IDE.TextEditor (grabFocus)
 import Control.Applicative ((<$>))
 import System.FilePath ((</>), dropFileName)
 import System.Exit (ExitCode(..))
+import IDE.Pane.Log (getLog)
+import Control.DeepSeq
+import qualified Data.Enumerator as E
+       (Step(..), run_, Iteratee(..), run)
+import qualified Data.Enumerator.List as EL
+       (foldM, head, dropWhile, isolate)
+import Data.Enumerator (($$), (>>==))
+import qualified Data.List as L ()
+import Control.Monad (foldM, when)
+import Control.Monad.Trans.Reader (ask)
+import Control.Monad.Trans.Class (MonadTrans(..))
+import Control.Monad.IO.Class (MonadIO(..))
 
 data GrepRecord = GrepRecord {
             file        :: FilePath
@@ -130,18 +145,42 @@ instance RecoverablePane IDEGrep GrepState IDEM where
         waitingGrep <- newEmptyMVar
         activeGrep <- newEmptyMVar
         let grep = IDEGrep {..}
-
+        let
+            gotoSource :: Bool -> IO Bool
+            gotoSource focus = do
+                sel <- getSelectionGrepRecord treeView grepStore
+                case sel of
+                    Just record -> reflectIDE (do
+                        case record of
+                            GrepRecord {file=f, line=l, parDir=Just pp} ->
+                                (goToSourceDefinition (pp </> f) $ Just $ Location l 0 l 0)
+                                    ?>>= (\b -> when focus $ grabFocus (sourceView b))
+                            _ -> return ()) ideR
+                    Nothing -> return ()
+                return True
         cid1 <- treeView `afterFocusIn`
             (\_ -> do reflectIDE (makeActive grep) ideR ; return True)
-        sel `onSelectionChanged` do
-            sel <- getSelectionGrepRecord treeView grepStore
-            case sel of
-                Just record -> reflectIDE (do
-                    case record of
-                        GrepRecord {file=f, line=l, parDir=Just pp} ->
-                            goToSourceDefinition (pp </> f) $ Just $ Location l 0 l 0
-                        _ -> return ()) ideR
-                Nothing -> return ()
+        cid2 <- treeView `onKeyPress`
+            (\e ->
+                case e of
+                    k@(Gdk.Key _ _ _ _ _ _ _ _ _ _)
+                        | Gdk.eventKeyName k == "Return"  -> do
+                            gotoSource True
+                        | Gdk.eventKeyName k == "Escape"  -> do
+                            reflectIDE (do
+                                lastActiveBufferPane ?>>= \paneName -> do
+                                    (PaneC pane) <- paneFromName paneName
+                                    makeActive pane
+                                    return ()
+                                triggerEventIDE StartFindInitial) ideR
+                            return True
+                            -- gotoSource True
+                        | otherwise -> do
+                            return False
+                    _ -> return False
+             )
+        sel `onSelectionChanged` (gotoSource False >> return ())
+
 
         return (Just grep,[ConnectC cid1])
 
@@ -202,55 +241,76 @@ grepDirectories regexString caseSensitive dirs = do
             putMVar (activeGrep grep) True
             takeMVar (waitingGrep grep)
 
-            postGUIAsync $ treeStoreClear store
+            postGUISync $ treeStoreClear store
 
             totalFound <- foldM (\a (dir, subDirs) -> do
                 nooneWaiting <- isEmptyMVar (waitingGrep grep)
                 found <- if nooneWaiting
                     then do
                         (output, pid) <- runTool "grep" ((if caseSensitive then [] else ["-i"])
-                            ++ ["-r", "-E", "-n", "--exclude=*~", regexString] ++ subDirs) (Just dir)
-                        reflectIDE (setGrepResults dir output) ideRef
+                            ++ ["-r", "-E", "-n", "-I", "--exclude=*~",
+#if !defined(darwin_HOST_OS)
+                                "--exclude-dir=.svn",
+                                "--exclude-dir=_darcs",
+                                "--exclude-dir=.git",
+#endif
+                                regexString] ++ subDirs) (Just dir)
+                        reflectIDE (do
+                            E.run_ $ output $$ do
+                                let max = 1000
+                                step <- EL.isolate (toInteger max) $$ setGrepResults dir
+                                case step of
+                                    E.Continue _ -> do
+                                        liftIO $ interruptProcessGroupOf pid
+                                        liftIO $ postGUISync $ do
+                                            nDir <- treeModelIterNChildren store Nothing
+                                            liftIO $ treeStoreChange store [nDir-1] (\r -> r{ context = "(Stoped Searching)" })
+                                            return ()
+                                        EL.dropWhile (const True)
+                                        return max
+                                    E.Yield n _ -> return n
+                                    _           -> return 0) ideRef
                     else return 0
                 return $ a + found) 0 dirs
 
             nooneWaiting <- isEmptyMVar (waitingGrep grep)
-            when nooneWaiting $ do
-                nDir <- postGUISync $ treeModelIterNChildren store Nothing
-                postGUIAsync $ treeStoreInsert store [] nDir $
-                    GrepRecord "Search Complete" totalFound "" Nothing
+            when nooneWaiting $ postGUISync $ do
+                nDir <- treeModelIterNChildren store Nothing
+                treeStoreInsert store [] nDir $ GrepRecord "Search Complete" totalFound "" Nothing
 
             takeMVar (activeGrep grep) >> return ()
     return ()
 
-setGrepResults :: FilePath -> [ToolOutput] -> IDEM Int
-setGrepResults dir output = do
-    grep <- getGrep Nothing
+setGrepResults :: FilePath -> E.Iteratee ToolOutput IDEM Int
+setGrepResults dir = do
+    ideRef <- lift ask
+    grep <- lift $ getGrep Nothing
+    log <- lift $ getLog
     let store = grepStore grep
         view  = treeView grep
-    ideRef <- ask
-    liftIO $ do
-        let (displayed, dropped) = splitAt 5000 output
-        forkIO $ do
-            let errors = filter isError output
-                    ++ if null dropped
-                        then []
-                        else [ToolError $ "Dropped " ++ show (length dropped) ++ " search results"]
-            unless (null errors) $ postGUISync $ reflectIDE (logOutput errors) ideRef
-            return ()
-        case catMaybes (map (process dir) displayed) of
-            []      -> return 0
-            results -> do
-                nDir <- postGUISync $ treeModelIterNChildren store Nothing
-                postGUIAsync $ treeStoreInsert store [] nDir $ GrepRecord dir 0 "" Nothing
-                forM_ (zip results [0..]) $ \(record, n) -> do
-                    nooneWaiting <- isEmptyMVar (waitingGrep grep)
-                    when nooneWaiting $ postGUIAsync $ do
-                        treeStoreInsert store [nDir] n record
-                        treeStoreChange store [nDir] (\r -> r{ line = n+1 }) >> return ()
-                        when (nDir == 0 && n == 0) $
-                            treeViewExpandAll view
-                return $ length results
+    nDir <- liftIO $ postGUISync $ do
+        nDir <- treeModelIterNChildren store Nothing
+        treeStoreInsert store [] nDir $ GrepRecord dir 0 "" Nothing
+        when (nDir == 0) (widgetGrabFocus view >> return())
+        return nDir
+    EL.foldM (\count line -> do
+        if isError line
+            then do
+                liftIO $ postGUISync $ reflectIDE (defaultLineLogger log line >> return ()) ideRef
+                return count
+            else do
+                case process dir line of
+                    Nothing     -> return count
+                    Just record -> liftIO $ do
+                        nooneWaiting <- isEmptyMVar (waitingGrep grep)
+                        when nooneWaiting $ postGUISync $ do
+                            parent <- treeModelGetIter store [nDir]
+                            n <- treeModelIterNChildren store parent
+                            treeStoreInsert store [nDir] n record
+                            treeStoreChange store [nDir] (\r -> r{ line = n+1 })
+                            when (nDir == 0 && n == 0) $
+                                treeViewExpandAll view
+                        return (count+1)) 0
     where
         process pp (ToolOutput line) =
             case parse grepLineParser "" line of
