@@ -14,7 +14,6 @@
 --
 ---------------------------------------------------------------------------------
 
-
 module IDE.Session (
     saveSession
 ,   saveSessionAs
@@ -26,7 +25,6 @@ module IDE.Session (
 ) where
 
 import Graphics.UI.Gtk hiding (showLayout)
-import Control.Monad.Reader
 import System.FilePath
 import qualified Data.Map as Map
 import Data.Maybe
@@ -48,6 +46,7 @@ import IDE.Pane.Preferences
 import IDE.Pane.PackageFlags
 import IDE.Pane.Search
 import IDE.Pane.Grep
+import IDE.Pane.Files
 import IDE.Pane.Breakpoints
 import IDE.Pane.Trace
 import IDE.Pane.Variables
@@ -59,6 +58,8 @@ import Control.Exception (SomeException(..))
 import IDE.Pane.Workspace (WorkspaceState(..))
 import IDE.Workspaces (workspaceOpenThis)
 import IDE.Completion (setCompletionSize)
+import Control.Monad.IO.Class (MonadIO(..))
+import Control.Monad (forM_, forM, when)
 
 
 -- ---------------------------------------------------------------------
@@ -78,6 +79,7 @@ data PaneState      =   BufferSt BufferState
                     |   PrefsSt PrefsState
                     |   FlagsSt FlagsState
                     |   SearchSt SearchState
+                    |   FilesSt FilesState
                     |   GrepSt GrepState
                     |   BreakpointsSt BreakpointsState
                     |   TraceSt TraceState
@@ -94,6 +96,7 @@ asPaneState s | isJust ((cast s) :: Maybe ModulesState)     =   ModulesSt (fromJ
 asPaneState s | isJust ((cast s) :: Maybe PrefsState)       =   PrefsSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe FlagsState)       =   FlagsSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe SearchState)      =   SearchSt (fromJust $ cast s)
+asPaneState s | isJust ((cast s) :: Maybe FilesState)       =   FilesSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe GrepState)        =   GrepSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe BreakpointsState) =   BreakpointsSt (fromJust $ cast s)
 asPaneState s | isJust ((cast s) :: Maybe TraceState)       =   TraceSt (fromJust $ cast s)
@@ -110,6 +113,7 @@ recover pp (ModulesSt p)        =   recoverState pp p >> return ()
 recover pp (PrefsSt p)          =   recoverState pp p >> return ()
 recover pp (FlagsSt p)          =   recoverState pp p >> return ()
 recover pp (SearchSt p)         =   recoverState pp p >> return ()
+recover pp (FilesSt p)          =   recoverState pp p >> return ()
 recover pp (GrepSt p)           =   recoverState pp p >> return ()
 recover pp (BreakpointsSt p)    =   recoverState pp p >> return ()
 recover pp (TraceSt p)          =   recoverState pp p >> return ()
